/*!\file window.c
 *
 * \brief Sphre en GL / GL4Dummies + Lumire positionnelle + diffus Phong + spculaire
 * \author Fars BELHADJ, amsi@ai.univ-paris8.fr
 * \date October 17 2017
 */

#include <GL4D/gl4du.h>
#include <GL4D/gl4dg.h>
#include <GL4D/gl4duw_SDL2.h>

/* Prototypes des fonctions statiques contenues dans ce fichier C */
static void         init(void);
static void         resize(int w, int h);
static void         keydown(int keycode);
static void         draw(void);
static void         quit(void);

/*!\brief dimensions de la fentre */
static int _windowWidth = 800, _windowHeight = 600;
/*!\brief identifiant de la gomtrie sphre */
static GLuint _sphere = 0;
/*!\brief identifiant du GLSL program */
static GLuint _pId = 0;
/*!\brief position de la lumire relativement  la sphre claire */
static GLfloat _lumPos0[4] = {-0.5, 0.5, 1.5, 1.0};


/*!\brief La fonction principale cr la fentre d'affichage,
 * initialise GL et les donnes, affecte les fonctions d'vnements et
 * lance la boucle principale d'affichage.
 */
int main(int argc, char ** argv) {
  if(!gl4duwCreateWindow(argc, argv, "GL4D", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, 
			 _windowWidth, _windowHeight, SDL_WINDOW_RESIZABLE | SDL_WINDOW_SHOWN))
    return 1;
  init();
  atexit(quit);
  gl4duwResizeFunc(resize);
  gl4duwKeyDownFunc(keydown);
  gl4duwDisplayFunc(draw);
  gl4duwMainLoop();
  return 0;
}

/*!\brief initialise les paramtres OpenGL */
static void init(void) {
  glClearColor(0.0f, 0.0f, 0.1f, 0.0f);
  _pId  = gl4duCreateProgram("<vs>shaders/basic.vs", "<fs>shaders/basic.fs", NULL);
  gl4duGenMatrix(GL_FLOAT, "modelViewMatrix");
  gl4duGenMatrix(GL_FLOAT, "projectionMatrix");
  resize(_windowWidth, _windowHeight);
  _sphere = gl4dgGenSpheref(3 /*longitudes*/, 3 /*latitudes*/);
}

/*!\brief Cette fonction paramtre la vue (viewport) OpenGL en
 * fonction des dimensions de la fentre.
 */
static void resize(int w, int h) {
  _windowWidth  = w;
  _windowHeight = h;
  glViewport(0, 0, _windowWidth, _windowHeight);
  gl4duBindMatrix("projectionMatrix");
  gl4duLoadIdentityf();
  gl4duFrustumf(-0.5, 0.5, -0.5 * _windowHeight / _windowWidth, 0.5 * _windowHeight / _windowWidth, 1.0, 1000.0);
  gl4duBindMatrix("modelViewMatrix");
}

static void keydown(int keycode) {
  GLint v[2];
  switch(keycode) {
  case 'w':
    glGetIntegerv(GL_POLYGON_MODE, v);
    if(v[0] == GL_FILL)
      glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    else
      glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    break;
  case SDLK_ESCAPE:
  case 'q':
    exit(0);
  default:
    break;
  }
}

/*!\brief Cette fonction dessine dans le contexte OpenGL actif.
 */
static void draw(void) {
  static GLfloat a0 = 0.0;
  static Uint32 t0 = 0;
  GLfloat dt = 0.0;
  GLfloat lumPos[4], *mat;
  Uint32 t;
  dt = ((t = SDL_GetTicks()) - t0) / 1000.0;
  t0 = t;
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glDisable(GL_CULL_FACE);
  glEnable(GL_DEPTH_TEST);
  glDisable(GL_BLEND);
  gl4duBindMatrix("modelViewMatrix");
  gl4duLoadIdentityf();
  gl4duTranslatef(0, 0, -3);
  mat = gl4duGetMatrixData();
  MMAT4XVEC4(lumPos, mat, _lumPos0);
  glUseProgram(_pId);
  gl4duPushMatrix();
  gl4duRotatef(a0, 0, 1, 0);
  glUniform4fv(glGetUniformLocation(_pId, "lumPos"), 1, lumPos);
  /* envoi de toutes les matrices stockes par GL4D */
  gl4duSendMatrices();
  /* dessiner une premire fois la sphre avec le _pId */
  gl4dgDraw(_sphere);
  gl4duPopMatrix();

  a0 += 360.0 * dt / (24.0 /* * 60.0 */);
}

/*!\brief appele au moment de sortir du programme (atexit), libre les lments utiliss */
static void quit(void) {
  gl4duClean(GL4DU_ALL);
}
